package ru.dbotthepony.mc.otm.client.model.entity;

import net.minecraft.client.model.HierarchicalModel;
import net.minecraft.client.model.geom.ModelLayerLocation;
import net.minecraft.client.model.geom.ModelPart;
import net.minecraft.client.model.geom.PartPose;
import net.minecraft.client.model.geom.builders.*;
import net.neoforged.neoforge.client.event.EntityRenderersEvent;
import org.jetbrains.annotations.NotNull;
import ru.dbotthepony.mc.otm.client.animation.ExoMechAnimation;
import ru.dbotthepony.mc.otm.entity.ExoMech;
import ru.dbotthepony.mc.otm.registry.MNames;

import static ru.dbotthepony.mc.otm.OverdriveThatMatters.loc;

public class ExoMechModel {
	public static final ModelLayerLocation LAYER_LOCATION = new ModelLayerLocation(loc(MNames.EXO_MECH), "main");

	private static HierarchicalModel<ExoMech> model;
	private static LayerDefinition def;



	public static HierarchicalModel<ExoMech> getModel() {
		if (def == null) {
			def = createBodyLayer();
		}

		if (model == null) {
			return model = new ExoMechHierarchicalModel<>(def.bakeRoot());
		}


		return model;
	}

	public static LayerDefinition createBodyLayer() {
		MeshDefinition meshdefinition = new MeshDefinition();
		PartDefinition partdefinition = meshdefinition.getRoot();

		PartDefinition root = partdefinition.addOrReplaceChild("root", CubeListBuilder.create(), PartPose.offset(0.0F, 27.0F, 0.0F));

		PartDefinition Cockpit = root.addOrReplaceChild("Cockpit", CubeListBuilder.create().texOffs(40, 37).addBox(12.0F, -13.2745F, 2.3512F, 2.0F, 7.0F, 7.0F, new CubeDeformation(0.0F))
			.texOffs(58, 37).addBox(-14.0F, -13.2745F, 2.3512F, 2.0F, 7.0F, 7.0F, new CubeDeformation(0.0F))
			.texOffs(90, 0).addBox(7.0F, -15.2745F, 0.3512F, 5.0F, 11.0F, 11.0F, new CubeDeformation(0.0F))
			.texOffs(58, 0).addBox(-12.0F, -15.2745F, 0.3512F, 5.0F, 11.0F, 11.0F, new CubeDeformation(0.0F))
			.texOffs(1, 74).addBox(-12.0F, -18.2745F, 2.3512F, 2.0F, 3.0F, 7.0F, new CubeDeformation(0.0F))
			.texOffs(1, 87).addBox(10.0F, -18.2745F, 2.3512F, 2.0F, 3.0F, 7.0F, new CubeDeformation(0.0F))
			.texOffs(122, 0).addBox(-7.0F, -16.2745F, 6.3512F, 14.0F, 13.0F, 6.0F, new CubeDeformation(0.0F))
			.texOffs(80, 22).addBox(-6.0F, -15.2745F, 12.3512F, 12.0F, 11.0F, 4.0F, new CubeDeformation(0.0F))
			.texOffs(79, 37).addBox(-5.0F, -3.2745F, 9.3512F, 10.0F, 6.0F, 6.0F, new CubeDeformation(0.0F))
			.texOffs(20, 53).addBox(-7.0F, -3.2745F, -9.6488F, 14.0F, 4.0F, 20.0F, new CubeDeformation(0.0F))
			.texOffs(93, 107).addBox(-11.0F, -5.2745F, -6.6488F, 4.0F, 5.0F, 16.0F, new CubeDeformation(0.0F))
			.texOffs(93, 86).addBox(7.0F, -5.2745F, -6.6488F, 4.0F, 5.0F, 16.0F, new CubeDeformation(0.0F))
			.texOffs(0, 0).addBox(-8.0F, -1.2745F, -10.6488F, 3.0F, 3.0F, 3.0F, new CubeDeformation(0.0F))
			.texOffs(0, 6).addBox(5.0F, -1.2745F, -10.6488F, 3.0F, 3.0F, 3.0F, new CubeDeformation(0.0F))
			.texOffs(112, 19).addBox(-6.0F, -4.0F, -5.0F, 12.0F, 1.0F, 11.0F, new CubeDeformation(0.0F)), PartPose.offset(0.0F, -27.7255F, -1.3512F));

		PartDefinition cube_r1 = Cockpit.addOrReplaceChild("cube_r1", CubeListBuilder.create().texOffs(0, 0).addBox(-9.0F, -12.0F, -11.0F, 18.0F, 14.0F, 22.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(0.0F, -7.0F, 0.0F, 0.5236F, 0.0F, 0.0F));

		PartDefinition Arm_L = Cockpit.addOrReplaceChild("Arm_L", CubeListBuilder.create(), PartPose.offset(12.0F, -10.0F, 6.0F));

		PartDefinition Body = root.addOrReplaceChild("Body", CubeListBuilder.create().texOffs(0, 36).addBox(-3.0F, -7.25F, -8.25F, 6.0F, 8.0F, 13.0F, new CubeDeformation(0.0F)), PartPose.offset(0.0F, -19.75F, 1.25F));

		PartDefinition thruster_r1 = Body.addOrReplaceChild("thruster_r1", CubeListBuilder.create().texOffs(0, 57).addBox(-3.0F, -4.0F, -3.0F, 6.0F, 8.0F, 6.0F, new CubeDeformation(-0.2F)), PartPose.offsetAndRotation(0.0F, -2.25F, 3.75F, 0.3491F, 0.0F, 0.0F));

		PartDefinition Leg_L = Body.addOrReplaceChild("Leg_L", CubeListBuilder.create().texOffs(60, 82).addBox(0.0F, -2.6667F, -5.3333F, 6.0F, 11.0F, 10.0F, new CubeDeformation(0.0F))
			.texOffs(83, 78).addBox(0.0F, -0.6667F, -5.3333F, 6.0F, 11.0F, 2.0F, new CubeDeformation(0.2F))
			.texOffs(60, 103).addBox(0.0F, 8.3333F, -3.3333F, 6.0F, 12.0F, 10.0F, new CubeDeformation(0.0F)), PartPose.offset(3.0F, -3.5833F, -1.9167F));

		PartDefinition thruster_r2 = Leg_L.addOrReplaceChild("thruster_r2", CubeListBuilder.create().texOffs(0, 114).addBox(-3.0F, -4.0F, -3.0F, 6.0F, 8.0F, 6.0F, new CubeDeformation(-0.2F)), PartPose.offsetAndRotation(3.0F, 11.3333F, 5.6667F, 0.7418F, 0.0F, 0.0F));

		PartDefinition Leg_R = Body.addOrReplaceChild("Leg_R", CubeListBuilder.create().texOffs(26, 103).addBox(-6.0F, 8.3333F, -3.3333F, 6.0F, 12.0F, 10.0F, new CubeDeformation(0.0F))
			.texOffs(26, 82).addBox(-6.0F, -2.6667F, -5.3333F, 6.0F, 11.0F, 10.0F, new CubeDeformation(0.0F))
			.texOffs(49, 78).addBox(-6.0F, -0.6667F, -5.3333F, 6.0F, 11.0F, 2.0F, new CubeDeformation(0.2F)), PartPose.offset(-3.0F, -3.5833F, -1.9167F));

		PartDefinition thruster_r3 = Leg_R.addOrReplaceChild("thruster_r3", CubeListBuilder.create().texOffs(0, 100).addBox(-3.0F, -4.0F, -3.0F, 6.0F, 8.0F, 6.0F, new CubeDeformation(-0.2F)), PartPose.offsetAndRotation(-3.0F, 11.3333F, 6.6667F, 0.7418F, 0.0F, 0.0F));

		model = null;
		return def = LayerDefinition.create(meshdefinition, 256, 128);
	}

	private static class ExoMechHierarchicalModel<T extends ExoMech> extends HierarchicalModel<T> {
		private final ModelPart root;
		private final ModelPart Body;
		public ExoMechHierarchicalModel(ModelPart root) {
			this.root = root.getChild("root");
			this.Body = this.root.getChild("Body");
		}

		@Override
		public void setupAnim(@NotNull T entity, float limbSwing, float limbSwingAmount, float ageInTicks, float netHeadYaw, float headPitch) {
			this.root().getAllParts().forEach(ModelPart::resetPose);

			this.Body.yRot = headPitch * ((float) Math.PI / 180F);
			this.Body.yRot = entity.getBodyRotation() * ((float) Math.PI / 180F);

			this.animateWalk(ExoMechAnimation.MOVE, limbSwing, limbSwingAmount, 1.8F, 1.0F);
			this.animate(entity.getIdleState(), ExoMechAnimation.IDLE, ageInTicks, 1.0F);
		}

		@Override
		public @NotNull ModelPart root() {
			return this.root;
		}
	}

	public static void register(EntityRenderersEvent.RegisterLayerDefinitions event) {
		event.registerLayerDefinition(LAYER_LOCATION, ExoMechModel::createBodyLayer);
	}
}
