package ru.dbotthepony.mc.otm.client.render.entity
import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.client.model.HierarchicalModel
import net.minecraft.client.model.HumanoidModel
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.util.ResourceLocation

import net.minecraft.client.model.PlayerModel
import net.minecraft.client.renderer.entity.EntityRendererProvider
import net.minecraft.client.renderer.entity.HumanoidMobRenderer
import net.minecraft.client.model.geom.ModelLayers
import net.minecraft.client.renderer.MultiBufferSource
import net.minecraft.client.renderer.RenderType
import net.minecraft.client.renderer.entity.layers.HumanoidArmorLayer
import net.minecraft.client.renderer.entity.layers.ItemInHandLayer
import net.minecraft.client.renderer.entity.layers.RenderLayer
import net.minecraft.client.renderer.texture.OverlayTexture
import net.minecraft.client.resources.model.ModelManager

import net.minecraft.resources.ResourceLocation
import net.minecraft.world.entity.EntityType
import net.minecraft.world.entity.Mob
import ru.dbotthepony.mc.otm.OverdriveThatMatters.loc
import ru.dbotthepony.mc.otm.entity.Loader


class RogueAndroidRenderer<T : Mob>(context: EntityRendererProvider.Context, private val entityType: EntityType<T>, private val androidTexture: String,private val modelManager: ModelManager) :
	HumanoidMobRenderer<T, PlayerModel<T>>(
		context, PlayerModel(context.bakeLayer(ModelLayers.PLAYER), false), 0.5f
	) {

	init {
		addLayer(ItemInHandLayer(this, context.itemInHandRenderer))
		this.addLayer(AndroidEmissiveLayer(this, androidTexture))

		addLayer(
			HumanoidArmorLayer(
				this,
				HumanoidModel(context.bakeLayer(ModelLayers.PLAYER_INNER_ARMOR)),
				HumanoidModel(context.bakeLayer(ModelLayers.PLAYER_OUTER_ARMOR)),
				modelManager
			)
		)
	}

	override fun getTextureLocation(entity: T): ResourceLocation {
		return ResourceLocation(OverdriveThatMatters.MOD_ID, "textures/entity/android/$androidTexture.png")
	}
}

class AndroidEmissiveLayer<T : Mob, M : HumanoidModel<T>>(
	renderer: HumanoidMobRenderer<T, M>,
	private val androidTexture: String
) : RenderLayer<T, M>(renderer) {

	private val emissive = loc("textures/entity/android/${androidTexture}_emissive.png")

	override fun render(
		poseStack: PoseStack,
		bufferSource: MultiBufferSource,
		packedLight: Int,
		entity: T,
		limbSwing: Float,
		limbSwingAmount: Float,
		partialTicks: Float,
		ageInTicks: Float,
		netHeadYaw: Float,
		headPitch: Float
	) {
		val buffer = bufferSource.getBuffer(RenderType.eyes(emissive))

		this.parentModel.renderToBuffer(
			poseStack,
			buffer,
			packedLight,
			OverlayTexture.NO_OVERLAY
		)
	}
}
