package ru.dbotthepony.mc.otm.compat.ironchest

import com.progwml6.ironchest.IronChests
import com.progwml6.ironchest.common.block.IronChestsTypes
import net.minecraft.core.BlockPos
import net.minecraft.core.registries.Registries
import net.minecraft.world.Container
import net.minecraft.world.flag.FeatureFlags
import net.minecraft.world.inventory.MenuType
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraft.world.level.block.state.BlockState
import net.neoforged.bus.api.IEventBus
import net.neoforged.fml.ModList
import net.neoforged.neoforge.capabilities.RegisterCapabilitiesEvent
import net.neoforged.neoforge.client.event.RegisterMenuScreensEvent
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.capability.IQuickStackContainer
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.compat.vanilla.VanillaChestScreen
import ru.dbotthepony.mc.otm.menu.MatteryMenuSlot
import ru.dbotthepony.mc.otm.menu.makeSlots
import ru.dbotthepony.mc.otm.registry.MDeferredRegister

internal val isIronChestLoaded by lazy {
	ModList.get().isLoaded(IronChests.MODID)
}

object IronChestMenuTypes {
	private val registrar = MDeferredRegister(Registries.MENU, OverdriveThatMatters.MOD_ID)

	val IRON by registrar.register("ironchest_iron") { MenuType(MatteryIronChestMenu::iron, FeatureFlags.VANILLA_SET) }
	val GOLD by registrar.register("ironchest_gold") { MenuType(MatteryIronChestMenu::gold, FeatureFlags.VANILLA_SET) }
	val DIAMOND by registrar.register("ironchest_diamond") { MenuType(MatteryIronChestMenu::diamond, FeatureFlags.VANILLA_SET) }
	val COPPER by registrar.register("ironchest_copper") { MenuType(MatteryIronChestMenu::copper, FeatureFlags.VANILLA_SET) }
	val CRYSTAL by registrar.register("ironchest_crystal") { MenuType(MatteryIronChestMenu::crystal, FeatureFlags.VANILLA_SET) }
	val OBSIDIAN by registrar.register("ironchest_obsidian") { MenuType(MatteryIronChestMenu::obsidian, FeatureFlags.VANILLA_SET) }
	val DIRT by registrar.register("ironchest_dirt") { MenuType(MatteryIronChestMenu::dirt, FeatureFlags.VANILLA_SET) }

	private fun provider(level: Level, pos: BlockPos, state: BlockState, blockEntity: BlockEntity?, context: Void?): IQuickStackContainer? {
		val container = blockEntity as? Container ?: return null
		return IQuickStackContainer.Simple(makeSlots(container, ::MatteryMenuSlot))
	}

	fun registerCapabilities(event: RegisterCapabilitiesEvent) {
		IronChestsTypes.entries.map { IronChestsTypes.get(it) }.forEach {
			event.registerBlock(MatteryCapability.QUICK_STACK_CONTAINER, ::provider, *it.toTypedArray())
		}
	}

	internal fun register(bus: IEventBus) {
		registrar.register(bus)
		bus.addListener(this::registerScreens)
		bus.addListener(this::registerCapabilities)
	}

	private fun registerScreens(event: RegisterMenuScreensEvent) {
		event.register(IRON, ::VanillaChestScreen)
		event.register(GOLD, ::VanillaChestScreen)
		event.register(DIAMOND, ::VanillaChestScreen)
		event.register(COPPER, ::VanillaChestScreen)
		event.register(CRYSTAL, ::VanillaChestScreen)
		event.register(OBSIDIAN, ::VanillaChestScreen)
		event.register(DIRT, ::VanillaChestScreen)
	}
}
