package ru.dbotthepony.mc.otm.compat.ironchest

import com.progwml6.ironchest.common.block.IronChestsTypes
import net.minecraft.world.Container
import net.minecraft.world.SimpleContainer
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.inventory.MenuType
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.Items
import ru.dbotthepony.mc.otm.compat.vanilla.AbstractVanillaChestMenu
import ru.dbotthepony.mc.otm.container.EnhancedContainer
import ru.dbotthepony.mc.otm.menu.MatteryMenuSlot
import ru.dbotthepony.mc.otm.menu.QuickMoveInput
import ru.dbotthepony.mc.otm.menu.makeSlots

class MatteryIronChestMenu(
	type: MenuType<*>, containerId: Int,
	inventory: Inventory, chestType: IronChestsTypes,
	container: Container = EnhancedContainer.Simple(chestType.size),
) : AbstractVanillaChestMenu(type, containerId, inventory, container) {
	override val columns = chestType.rowLength
	override val rows = chestType.size / chestType.rowLength

	override val containerSlots = if (chestType == IronChestsTypes.DIRT) makeSlots(container, ::MDirtChestSlot) else makeSlots(container, ::MatteryMenuSlot)

	init {
		container.startOpen(player)
		addStorageSlot(containerSlots)
		addInventorySlots()
	}

	override val quickMoveToStorage = QuickMoveInput.create(this, playerCombinedInventorySlots, containerSlots)
	override val quickMoveFromStorage = QuickMoveInput.create(this, containerSlots, playerInventorySlots, false)

	class MDirtChestSlot(container: Container, slot: Int) : MatteryMenuSlot(container, slot) {
		override fun mayPlace(stack: ItemStack): Boolean = stack.isEmpty || stack.`is`(Items.DIRT)
	}

	companion object {
		@JvmStatic
		@JvmOverloads
		fun iron(containerId: Int, inventory: Inventory, container: Container = SimpleContainer(IronChestsTypes.IRON.size), chestType: IronChestsTypes = IronChestsTypes.IRON): MatteryIronChestMenu {
			return MatteryIronChestMenu(IronChestMenuTypes.IRON, containerId, inventory, chestType, container)
		}

		@JvmStatic
		@JvmOverloads
		fun gold(containerId: Int, inventory: Inventory, container: Container = SimpleContainer(IronChestsTypes.GOLD.size), chestType: IronChestsTypes = IronChestsTypes.GOLD): MatteryIronChestMenu {
			return MatteryIronChestMenu(IronChestMenuTypes.GOLD, containerId, inventory, chestType, container)
		}

		@JvmStatic
		@JvmOverloads
		fun diamond(containerId: Int, inventory: Inventory, container: Container = SimpleContainer(IronChestsTypes.DIAMOND.size), chestType: IronChestsTypes = IronChestsTypes.DIAMOND): MatteryIronChestMenu {
			return MatteryIronChestMenu(IronChestMenuTypes.DIAMOND, containerId, inventory, chestType, container)
		}

		@JvmStatic
		@JvmOverloads
		fun copper(containerId: Int, inventory: Inventory, container: Container = SimpleContainer(IronChestsTypes.COPPER.size), chestType: IronChestsTypes = IronChestsTypes.COPPER): MatteryIronChestMenu {
			return MatteryIronChestMenu(IronChestMenuTypes.COPPER, containerId, inventory, chestType, container)
		}

		@JvmStatic
		@JvmOverloads
		fun crystal(containerId: Int, inventory: Inventory, container: Container = SimpleContainer(IronChestsTypes.CRYSTAL.size), chestType: IronChestsTypes = IronChestsTypes.CRYSTAL): MatteryIronChestMenu {
			return MatteryIronChestMenu(IronChestMenuTypes.CRYSTAL, containerId, inventory, chestType, container)
		}

		@JvmStatic
		@JvmOverloads
		fun obsidian(containerId: Int, inventory: Inventory, container: Container = SimpleContainer(IronChestsTypes.OBSIDIAN.size), chestType: IronChestsTypes = IronChestsTypes.OBSIDIAN): MatteryIronChestMenu {
			return MatteryIronChestMenu(IronChestMenuTypes.OBSIDIAN, containerId, inventory, chestType, container)
		}

		@JvmStatic
		@JvmOverloads
		fun dirt(containerId: Int, inventory: Inventory, container: Container = SimpleContainer(IronChestsTypes.DIRT.size), chestType: IronChestsTypes = IronChestsTypes.DIRT): MatteryIronChestMenu {
			return MatteryIronChestMenu(IronChestMenuTypes.DIRT, containerId, inventory, chestType, container)
		}
	}
}
