package ru.dbotthepony.mc.otm.compat.ironshulkerbox

import com.progwml6.ironshulkerbox.IronShulkerBoxes
import com.progwml6.ironshulkerbox.common.block.IronShulkerBoxesTypes
import net.minecraft.core.BlockPos
import net.minecraft.core.registries.Registries
import net.minecraft.world.Container
import net.minecraft.world.flag.FeatureFlags
import net.minecraft.world.inventory.MenuType
import net.minecraft.world.item.DyeColor
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraft.world.level.block.state.BlockState
import net.neoforged.bus.api.IEventBus
import net.neoforged.fml.ModList
import net.neoforged.neoforge.capabilities.RegisterCapabilitiesEvent
import net.neoforged.neoforge.client.event.RegisterMenuScreensEvent
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.capability.IQuickStackContainer
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.compat.ironshulkerbox.MatteryIronShulkerBoxMenu.Slot
import ru.dbotthepony.mc.otm.compat.vanilla.VanillaChestScreen
import ru.dbotthepony.mc.otm.menu.makeSlots
import ru.dbotthepony.mc.otm.registry.MDeferredRegister
import kotlin.collections.toTypedArray

internal val isIronShulkersLoaded by lazy {
	ModList.get().isLoaded(IronShulkerBoxes.MODID)
}

object IronShulkersMenuTypes {
	private val registrar = MDeferredRegister(Registries.MENU, OverdriveThatMatters.MOD_ID)

	val IRON by registrar.register("ironshulkerbox_iron") { MenuType(MatteryIronShulkerBoxMenu::iron, FeatureFlags.VANILLA_SET) }
	val GOLD by registrar.register("ironshulkerbox_gold") { MenuType(MatteryIronShulkerBoxMenu::gold, FeatureFlags.VANILLA_SET) }
	val DIAMOND by registrar.register("ironshulkerbox_diamond") { MenuType(MatteryIronShulkerBoxMenu::diamond, FeatureFlags.VANILLA_SET) }
	val COPPER by registrar.register("ironshulkerbox_copper") { MenuType(MatteryIronShulkerBoxMenu::copper, FeatureFlags.VANILLA_SET) }
	val CRYSTAL by registrar.register("ironshulkerbox_crystal") { MenuType(MatteryIronShulkerBoxMenu::crystal, FeatureFlags.VANILLA_SET) }
	val OBSIDIAN by registrar.register("ironshulkerbox_obsidian") { MenuType(MatteryIronShulkerBoxMenu::obsidian, FeatureFlags.VANILLA_SET) }

	private fun provider(level: Level, pos: BlockPos, state: BlockState, blockEntity: BlockEntity?, context: Void?): IQuickStackContainer? {
		val container = blockEntity as? Container ?: return null
		return IQuickStackContainer.Simple(makeSlots(container, ::Slot))
	}

	fun registerCapabilities(event: RegisterCapabilitiesEvent) {
		val colors = DyeColor.entries.toMutableList<DyeColor?>().also { it.add(0, null) }

		IronShulkerBoxesTypes.entries.forEach { type ->
			event.registerBlock(MatteryCapability.QUICK_STACK_CONTAINER, ::provider, *colors.map { IronShulkerBoxesTypes.get(type, it) }.toTypedArray())
		}
	}

	internal fun register(bus: IEventBus) {
		registrar.register(bus)
		bus.addListener(this::registerScreens)
		bus.addListener(this::registerCapabilities)
	}

	private fun registerScreens(event: RegisterMenuScreensEvent) {
		event.register(IRON, ::VanillaChestScreen)
		event.register(GOLD, ::VanillaChestScreen)
		event.register(DIAMOND, ::VanillaChestScreen)
		event.register(COPPER, ::VanillaChestScreen)
		event.register(CRYSTAL, ::VanillaChestScreen)
		event.register(OBSIDIAN, ::VanillaChestScreen)
	}
}
