package ru.dbotthepony.mc.otm.compat.ironshulkerbox

import com.progwml6.ironshulkerbox.common.block.IronShulkerBoxesTypes
import net.minecraft.world.Container
import net.minecraft.world.SimpleContainer
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.inventory.MenuType
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.compat.vanilla.AbstractVanillaChestMenu
import ru.dbotthepony.mc.otm.container.EnhancedContainer
import ru.dbotthepony.mc.otm.menu.MatteryMenuSlot
import ru.dbotthepony.mc.otm.menu.QuickMoveInput
import ru.dbotthepony.mc.otm.menu.makeSlots

class MatteryIronShulkerBoxMenu(
	type: MenuType<*>, containerId: Int,
	inventory: Inventory, boxType: IronShulkerBoxesTypes,
	container: Container = EnhancedContainer.Simple(boxType.size)
) : AbstractVanillaChestMenu(type, containerId, inventory, container) {
	override val rows: Int = boxType.size / boxType.rowLength
	override val columns: Int = boxType.rowLength

	override val containerSlots = makeSlots(container, ::Slot)

	init {
		container.startOpen(player)
		addStorageSlot(containerSlots)
		addInventorySlots()
	}

	override val quickMoveToStorage = QuickMoveInput.create(this, playerCombinedInventorySlots, containerSlots)
	override val quickMoveFromStorage = QuickMoveInput.create(this, containerSlots, playerInventorySlots, false)

	class Slot(container: Container, slot: Int) : MatteryMenuSlot(container, slot) {
		override fun mayPlace(stack: ItemStack): Boolean {
			return super.mayPlace(stack) && stack.item.canFitInsideContainerItems()
		}
	}

	companion object {
		@JvmStatic
		@JvmOverloads
		fun iron(containerId: Int, inventory: Inventory, container: Container = SimpleContainer(IronShulkerBoxesTypes.IRON.size), boxType: IronShulkerBoxesTypes = IronShulkerBoxesTypes.IRON): MatteryIronShulkerBoxMenu {
			return MatteryIronShulkerBoxMenu(IronShulkersMenuTypes.IRON, containerId, inventory, boxType, container)
		}

		@JvmStatic
		@JvmOverloads
		fun gold(containerId: Int, inventory: Inventory, container: Container = SimpleContainer(IronShulkerBoxesTypes.GOLD.size), boxType: IronShulkerBoxesTypes = IronShulkerBoxesTypes.GOLD): MatteryIronShulkerBoxMenu {
			return MatteryIronShulkerBoxMenu(IronShulkersMenuTypes.GOLD, containerId, inventory, boxType, container)
		}

		@JvmStatic
		@JvmOverloads
		fun diamond(containerId: Int, inventory: Inventory, container: Container = SimpleContainer(IronShulkerBoxesTypes.DIAMOND.size), boxType: IronShulkerBoxesTypes = IronShulkerBoxesTypes.DIAMOND): MatteryIronShulkerBoxMenu {
			return MatteryIronShulkerBoxMenu(IronShulkersMenuTypes.DIAMOND, containerId, inventory, boxType, container)
		}

		@JvmStatic
		@JvmOverloads
		fun copper(containerId: Int, inventory: Inventory, container: Container = SimpleContainer(IronShulkerBoxesTypes.COPPER.size), boxType: IronShulkerBoxesTypes = IronShulkerBoxesTypes.COPPER): MatteryIronShulkerBoxMenu {
			return MatteryIronShulkerBoxMenu(IronShulkersMenuTypes.COPPER, containerId, inventory, boxType, container)
		}

		@JvmStatic
		@JvmOverloads
		fun crystal(containerId: Int, inventory: Inventory, container: Container = SimpleContainer(IronShulkerBoxesTypes.CRYSTAL.size), boxType: IronShulkerBoxesTypes = IronShulkerBoxesTypes.CRYSTAL): MatteryIronShulkerBoxMenu {
			return MatteryIronShulkerBoxMenu(IronShulkersMenuTypes.CRYSTAL, containerId, inventory, boxType, container)
		}

		@JvmStatic
		@JvmOverloads
		fun obsidian(containerId: Int, inventory: Inventory, container: Container = SimpleContainer(IronShulkerBoxesTypes.OBSIDIAN.size), boxType: IronShulkerBoxesTypes = IronShulkerBoxesTypes.OBSIDIAN): MatteryIronShulkerBoxMenu {
			return MatteryIronShulkerBoxMenu(IronShulkersMenuTypes.OBSIDIAN, containerId, inventory, boxType, container)
		}
	}
}
