package ru.dbotthepony.mc.otm.entity

import net.minecraft.network.syncher.SynchedEntityData
import net.minecraft.sounds.SoundEvent
import net.minecraft.sounds.SoundEvents
import net.minecraft.world.InteractionHand
import net.minecraft.world.InteractionResult
import net.minecraft.world.entity.*
import net.minecraft.world.entity.ai.attributes.AttributeSupplier
import net.minecraft.world.entity.ai.attributes.Attributes
import net.minecraft.world.entity.monster.Monster
import net.minecraft.world.entity.player.Player
import net.minecraft.world.level.Level
import kotlin.math.atan2
import kotlin.math.cos
import kotlin.math.sin

class ExoMech(type: EntityType<out PathfinderMob>, level: Level) : PathfinderMob(type, level) {

	val idleState = AnimationState()

	private var bodyRotation: Float = 0.0f
	private var lastBodyRot: Float = 0.0f
	private var bodyRotation2: Float = 0.0f

	companion object {
		fun createAttributes(): AttributeSupplier.Builder {
			return Monster.createMonsterAttributes()
				.add(Attributes.MAX_HEALTH, 60.0)
				.add(Attributes.MOVEMENT_SPEED, 1.2)
				.add(Attributes.KNOCKBACK_RESISTANCE, 0.7)
				.add(Attributes.STEP_HEIGHT, 1.0)
		}
	}

	init {
		idleState.start(tickCount)
	}

	override fun defineSynchedData(builder: SynchedEntityData.Builder) {
		super.defineSynchedData(builder)
	}

	override fun getControllingPassenger(): LivingEntity? {
		return firstPassenger as? LivingEntity
	}

	override fun tick() {
		super.tick()

		val rider = controllingPassenger
		if (rider is Player) {
			yBodyRot = rider.yHeadRot
		}


		//separate rotation for the waist, supposed to turn towards where the mech is moving, not where it's looking
		val movementVec = this.deltaMovement
		if (movementVec.length() > 0.01) {
			val yawRad = Math.toRadians(-yRot.toDouble())
			val sin = sin(yawRad)
			val cos = cos(yawRad)

			val movedX = movementVec.x * cos - movementVec.z * sin
			val movedZ = movementVec.x * sin + movementVec.z * cos

			bodyRotation = (atan2(-movedX, movedZ) * (180 / Math.PI)).toFloat()
			bodyRotation2 = lerpAngle(lastBodyRot, bodyRotation, 0.5f)
			lastBodyRot = bodyRotation2
		}
	}

	private fun lerpAngle(start: Float, end: Float, factor: Float): Float {
		var difference = (end - start) % 360
		if (difference > 180) difference -= 360
		if (difference < -180) difference += 360
		return start + difference * factor
	}

	fun getBodyRotation(): Float = bodyRotation2

	override fun mobInteract(player: Player, hand: InteractionHand): InteractionResult {
		if (!this.level().isClientSide) {
			player.startRiding(this)
			return InteractionResult.SUCCESS
		}
		return InteractionResult.CONSUME
	}

	override fun isPushable(): Boolean = false

	override fun isControlledByLocalInstance(): Boolean = this.controllingPassenger is Player

	override fun getHurtSound(damageSource: net.minecraft.world.damagesource.DamageSource): SoundEvent {
		return SoundEvents.IRON_GOLEM_HURT
	}

	override fun getDeathSound(): SoundEvent {
		return SoundEvents.IRON_GOLEM_DEATH
	}

	override fun playStepSound(pos: net.minecraft.core.BlockPos, blockState: net.minecraft.world.level.block.state.BlockState) {
		this.playSound(SoundEvents.IRON_GOLEM_STEP, 1.0f, 1.0f)
	}
}
